@@ -20,7 +20,9 @@
  * support other XAuth types and possibly DECnet... */
 
 #include <config.h>
-#include <gnome.h>
+#include <libgnome/libgnome.h>
+#include <sys/types.h>
+#include <unistd.h>
 #include <sys/stat.h>
 #include <netdb.h> 
 #include <netinet/in.h>
@@ -34,9 +36,6 @@
 #include "filecheck.h"
 #include "auth.h"
 
-static const gchar RCSid[]="$Id: auth.c,v 1.21 2001/10/28 19:13:42 jirka Exp $";
-
-
 /* Local prototypes */
 static void gdm_auth_purge (GdmDisplay *d, FILE *af);
 
@@ -59,8 +58,6 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 	if (!d)
 		return FALSE;
 
-	dispnum = g_strdup_printf ("%d", d->dispnum);
-
 	xa = malloc (sizeof (Xauth));
 
 	if (xa == NULL)
@@ -72,10 +69,16 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 		free (xa);
 		return FALSE;
 	}
+
+
 	memcpy (xa->address, addr, addrlen);
 	xa->address_length = addrlen;
+
+	dispnum = g_strdup_printf ("%d", d->dispnum);
 	xa->number = strdup (dispnum);
 	xa->number_length = strlen (dispnum);
+	g_free (dispnum);
+
 	xa->name = strdup ("MIT-MAGIC-COOKIE-1");
 	xa->name_length = strlen ("MIT-MAGIC-COOKIE-1");
 	xa->data = malloc (16);
@@ -95,8 +98,6 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 
 	d->auths = g_slist_append (d->auths, xa);
 
-	g_free (dispnum);
-
 	return TRUE;
 }
 
@@ -140,7 +141,7 @@ gdm_auth_secure_display (GdmDisplay *d)
 	    d->authfile = g_strconcat (GdmUserAuthFB, "/.gdmXXXXXX", NULL);
 
 	    umask (077);
-	    authfd = mkstemp (d->authfile);
+	    authfd = g_mkstemp (d->authfile);
 	    umask (022);
 
 	    if (authfd == -1) {
@@ -313,7 +314,7 @@ gdm_auth_secure_display (GdmDisplay *d)
     fclose (af);
     if (af_gdm != NULL)
 	    fclose (af_gdm);
-    ve_setenv ("XAUTHORITY", d->authfile, TRUE);
+    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
 
     if (GdmDebug)
 	    gdm_debug ("gdm_auth_secure_display: Setting up access for %s - %d entries", 
@@ -364,7 +365,7 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
 	/* No go. Let's create a fallback file in GdmUserAuthFB (/tmp) */
 	d->authfb = TRUE;
 	d->userauth = g_strconcat (GdmUserAuthFB, "/.gdmXXXXXX", NULL);
-	authfd = mkstemp (d->userauth);
+	authfd = g_mkstemp (d->userauth);
 
 	if (authfd == -1) {
 	    gdm_error (_("gdm_auth_user_add: Could not open cookie file %s"), d->userauth);
@@ -446,7 +447,7 @@ void
 gdm_auth_user_remove (GdmDisplay *d, uid_t user)
 {
     FILE *af;
-    const gchar *authfile;
+    gchar *authfile;
     gchar *authdir;
 
     if (!d || !d->userauth)
@@ -463,22 +464,24 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
 	return;
     }
 
-    authfile = g_basename (d->userauth);
-    authdir = g_dirname (d->userauth);
+    authfile = g_path_get_basename (d->userauth);
+    authdir = g_path_get_dirname (d->userauth);
 
     /* Now, the cookie file could be owned by a malicious user who
      * decided to concatenate something like his entire MP3 collection
      * to it. So we better play it safe... */
 
-    if (! gdm_file_check ("gdm_auth_user_remove", user, authdir, authfile, 
-			  TRUE, GdmUserMaxFile, GdmRelaxPerms)) {
+    if ( ! gdm_file_check ("gdm_auth_user_remove", user, authdir, authfile, 
+			   TRUE, GdmUserMaxFile, GdmRelaxPerms)) {
 	    g_free (authdir);
+	    g_free (authfile);
 	    gdm_error (_("gdm_auth_user_remove: Ignoring suspiciously looking cookie file %s"), d->userauth);
 
 	    return; 
     }
 
     g_free (authdir);
+    g_free (authfile);
 
     /* Lock user's cookie jar and open it for writing */
     if (XauLockAuth (d->userauth, 3, 3, 0) != LOCK_SUCCESS)
