@@ -68,7 +68,6 @@ GSList *xservers = NULL;	/* List of x server definitions */
 gint high_display_num = 0;	/* Highest local non-flexi display */
 gint sessions = 0;		/* Number of remote sessions */
 gint flexi_servers = 0;		/* Number of flexi servers */
-sigset_t sysmask;		/* Inherited system signal mask */
 uid_t GdmUserId;		/* Userid under which gdm should run */
 gid_t GdmGroupId;		/* Groupid under which gdm should run */
 pid_t extra_process = -1;	/* An extra process.  Used for quickie 
@@ -90,6 +89,8 @@ gboolean gdm_emergency_server = FALSE;
 
 gboolean gdm_first_login = TRUE;
 
+int gdm_in_signal = 0;
+
 /* Configuration options */
 gchar *GdmUser = NULL;
 gchar *GdmGroup = NULL;
@@ -106,6 +107,7 @@ gboolean GdmBrowser = FALSE;
 gchar *GdmGlobalFaceDir = NULL;
 gint GdmXineramaScreen = 0;
 gchar *GdmGreeter = NULL;
+gchar *GdmRemoteGreeter = NULL;
 gchar *GdmChooser = NULL;
 gchar *GdmLogDir = NULL;
 gchar *GdmDisplayInit = NULL;
@@ -157,6 +159,8 @@ char **stored_argv = NULL;
 int stored_argc = 0;
 char *stored_path = NULL;
 
+static time_t config_file_mtime = 0;
+
 static gboolean gdm_restart_mode = FALSE;
 
 static GMainLoop *main_loop = NULL;
@@ -206,8 +210,12 @@ gdm_config_parse (void)
     displays = NULL;
     high_display_num = 0;
 
-    if (stat (GDM_CONFIG_FILE, &statbuf) == -1)
-	gdm_error (_("gdm_config_parse: No configuration file: %s. Using defaults."), GDM_CONFIG_FILE);
+    if (stat (GDM_CONFIG_FILE, &statbuf) == -1) {
+	    gdm_error (_("%s: No configuration file: %s. Using defaults."),
+		       "gdm_config_parse", GDM_CONFIG_FILE);
+    } else {
+	    config_file_mtime = statbuf.st_mtime;
+    }
 
     /* Parse configuration options */
     gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
@@ -219,6 +227,7 @@ gdm_config_parse (void)
     GdmAutomaticLogin = gnome_config_get_string (GDM_KEY_AUTOMATICLOGIN);
     GdmAlwaysRestartServer = gnome_config_get_bool (GDM_KEY_ALWAYSRESTARTSERVER);
     GdmGreeter = gnome_config_get_string (GDM_KEY_GREETER);
+    GdmRemoteGreeter = gnome_config_get_string (GDM_KEY_REMOTEGREETER);
     GdmGroup = gnome_config_get_string (GDM_KEY_GROUP);
     GdmHalt = gnome_config_get_string (GDM_KEY_HALT);
     GdmKillInitClients = gnome_config_get_bool (GDM_KEY_KILLIC);
@@ -273,9 +282,20 @@ gdm_config_parse (void)
     GdmWilling = gnome_config_get_string (GDM_KEY_WILLING);    
 
     GdmStandardXServer = gnome_config_get_string (GDM_KEY_STANDARD_XSERVER);    
-    if (ve_string_empty (GdmStandardXServer)) {
-	    g_free (GdmStandardXServer);
-	    GdmStandardXServer = g_strdup ("/usr/bin/X11/X");
+    if (ve_string_empty (GdmStandardXServer) ||
+	access (GdmStandardXServer, X_OK) != 0) {
+	    gdm_info (_("%s: Standard X server not found, trying alternatives"),
+			"gdm_config_parse");
+	    if (access ("/usr/X11R6/bin/X", X_OK) == 0) {
+		    g_free (GdmStandardXServer);
+		    GdmStandardXServer = g_strdup ("/usr/X11R6/bin/X");
+	    } else if (access ("/opt/X11R6/bin/X", X_OK) == 0) {
+		    g_free (GdmStandardXServer);
+		    GdmStandardXServer = g_strdup ("/opt/X11R6/bin/X");
+	    } else if (ve_string_empty (GdmStandardXServer)) {
+		    g_free (GdmStandardXServer);
+		    GdmStandardXServer = g_strdup ("/usr/bin/X11/X");
+	    }
     }
     GdmFlexibleXServers = gnome_config_get_int (GDM_KEY_FLEXIBLE_XSERVERS);    
     GdmXnest = gnome_config_get_string (GDM_KEY_XNEST);    
@@ -290,7 +310,7 @@ gdm_config_parse (void)
 
 #ifndef HAVE_LIBXDMCP
     if (GdmXdmcp) {
-	    gdm_info (_("gdm_config_parse: XDMCP was enabled while there is no XDMCP support, turning it off"));
+	    gdm_info (_("%s: XDMCP was enabled while there is no XDMCP support, turning it off"), "gdm_config_parse");
 	    GdmXdmcp = FALSE;
     }
 #endif
@@ -303,7 +323,7 @@ gdm_config_parse (void)
 
     if (GdmAutomaticLogin != NULL &&
 	strcmp (GdmAutomaticLogin, "root") == 0) {
-	    gdm_info (_("gdm_config_parse: Root cannot be autologged in, turing off automatic login"));
+	    gdm_info (_("%s: Root cannot be autologged in, turing off automatic login"), "gdm_config_parse");
 	    g_free (GdmAutomaticLogin);
 	    GdmAutomaticLogin = NULL;
     }
@@ -316,13 +336,13 @@ gdm_config_parse (void)
 
     if (GdmTimedLogin != NULL &&
 	strcmp (GdmTimedLogin, "root") == 0) {
-	    gdm_info (_("gdm_config_parse: Root cannot be autologged in, turing off timed login"));
+	    gdm_info (_("%s: Root cannot be autologged in, turing off timed login"), "gdm_config_parse");
 	    g_free (GdmTimedLogin);
 	    GdmTimedLogin = NULL;
     }
 
     if (GdmTimedLoginDelay < 5) {
-	    gdm_info (_("gdm_config_parse: TimedLoginDelay less then 5, so I will just use 5."));
+	    gdm_info (_("%s: TimedLoginDelay less then 5, so I will just use 5."), "gdm_config_parse");
 	    GdmTimedLoginDelay = 5;
     }
 
@@ -332,19 +352,23 @@ gdm_config_parse (void)
 
     /* Prerequisites */ 
     if (ve_string_empty (GdmGreeter)) {
-	    gdm_error (_("gdm_config_parse: No greeter specified."));
+	    gdm_error (_("%s: No greeter specified."), "gdm_config_parse");
+    }
+    if (ve_string_empty (GdmRemoteGreeter)) {
+	    gdm_error (_("%s: No remote greeter specified."), "gdm_config_parse");
     }
 
     if (ve_string_empty (GdmServAuthDir)) {
-	    gdm_text_message_dialog (_("No daemon/ServAuthDir specified in the configuration file"));
-	    gdm_fail (_("gdm_config_parse: No authdir specified."));
+	    gdm_text_message_dialog
+		    (_("No daemon/ServAuthDir specified in the configuration file"));
+	    gdm_fail (_("%s: No authdir specified."), "gdm_config_parse");
     }
 
     if (ve_string_empty (GdmLogDir))
 	GdmLogDir = GdmServAuthDir;
 
     if (ve_string_empty (GdmSessDir))
-	gdm_error (_("gdm_config_parse: No sessions directory specified."));
+	gdm_error (_("%s: No sessions directory specified."), "gdm_config_parse");
 
     /* Find server definitions */
     iter = gnome_config_init_iterator_sections ("=" GDM_CONFIG_FILE "=/");
@@ -430,7 +454,7 @@ gdm_config_parse (void)
 		    if (disp_num > high_display_num)
 			    high_display_num = disp_num;
 	    } else {
-		    gdm_info (_("gdm_config_parse: Invalid server line in config file. Ignoring!"));
+		    gdm_info (_("%s: Invalid server line in config file. Ignoring!"), "gdm_config_parse");
 	    }
 	    g_free (k);
 	    g_free (v);
@@ -444,14 +468,18 @@ gdm_config_parse (void)
 		    server = GdmStandardXServer;
 	    } else if (access ("/usr/bin/X11/X", X_OK) == 0) {
 		    server = "/usr/bin/X11/X";
+	    } else if (access ("/usr/X11R6/bin/X", X_OK) == 0) {
+		    server = "/usr/X11R6/bin/X";
+	    } else if (access ("/opt/X11R6/bin/X", X_OK) == 0) {
+		    server = "/opt/X11R6/bin/X";
 	    }
 	    /* yay, we can add a backup emergency server */
 	    if (server != NULL) {
 		    int num = gdm_get_free_display (0 /* start */,
 						    0 /* server uid */);
-		    gdm_error (_("%s: Xdmcp disabled and no local servers defined. Adding /usr/bin/X11/X on :%d to allow configuration!"),
+		    gdm_error (_("%s: XDMCP disabled and no local servers defined. Adding %s on :%d to allow configuration!"),
 			       "gdm_config_parse",
-			       num);
+			       server, num);
 
 		    gdm_emergency_server = TRUE;
 		    displays = g_slist_append
@@ -463,14 +491,15 @@ gdm_config_parse (void)
 		    g_free (GdmTimedLogin);
 		    GdmTimedLogin = NULL;
 	    } else {
-		    char *s = g_strdup_printf (_("Xdmcp is disabled and gdm "
-						 "cannot find any local server "
-						 "to start.  Aborting!  Please "
-						 "correct the configuration %s "
-						 "and restart gdm."),
-					       GDM_CONFIG_FILE);
+		    char *s = g_strdup_printf
+			    (_("XDMCP is disabled and gdm "
+			       "cannot find any local server "
+			       "to start.  Aborting!  Please "
+			       "correct the configuration %s "
+			       "and restart gdm."),
+			     GDM_CONFIG_FILE);
 		    gdm_text_message_dialog (s);
-		    gdm_fail (_("gdm_config_parse: Xdmcp disabled and no local servers defined. Aborting!"));
+		    gdm_fail (_("%s: XDMCP disabled and no local servers defined. Aborting!"), "gdm_config_parse");
 	    }
     }
 
@@ -478,61 +507,65 @@ gdm_config_parse (void)
     pwent = getpwnam (GdmUser);
 
     if (pwent == NULL) {
-	    gdm_error (_("gdm_config_parse: Can't find the gdm user (%s). Trying 'nobody'!"), GdmUser);
+	    gdm_error (_("%s: Can't find the gdm user (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmUser);
 	    GdmUser = g_strdup ("nobody");
 	    pwent = getpwnam (GdmUser);
     }
 
     if (pwent == NULL) {
-	    char *s = g_strdup_printf (_("The gdm user does not exist. "
-					 "Please correct gdm configuration %s "
-					 "and restart gdm."),
-				       GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("The gdm user does not exist. "
+		       "Please correct gdm configuration %s "
+		       "and restart gdm."),
+		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: Can't find the gdm user (%s). Aborting!"), GdmUser);
+	    gdm_fail (_("%s: Can't find the gdm user (%s). Aborting!"), "gdm_config_parse", GdmUser);
     } else {
 	    GdmUserId = pwent->pw_uid;
     }
 
     if (GdmUserId == 0) {
-	    char *s = g_strdup_printf (_("The gdm user is set to be root, but "
-					 "this is not allowed since it can "
-					 "pose a security risk.  Please "
-					 "correct gdm configuration %s and "
-					 "restart gdm."), GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("The gdm user is set to be root, but "
+		       "this is not allowed since it can "
+		       "pose a security risk.  Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: The gdm user should not be root. Aborting!"));
+	    gdm_fail (_("%s: The gdm user should not be root. Aborting!"), "gdm_config_parse");
     }
 
     grent = getgrnam (GdmGroup);
 
     if (grent == NULL) {
-	    gdm_error (_("gdm_config_parse: Can't find the gdm group (%s). Trying 'nobody'!"), GdmUser);
+	    gdm_error (_("%s: Can't find the gdm group (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmGroup);
 	    GdmGroup = g_strdup ("nobody");
 	    pwent = getpwnam (GdmUser);
     }
 
     if (grent == NULL) {
-	    char *s = g_strdup_printf (_("The gdm group does not exist. "
-					 "Please correct gdm configuration %s "
-					 "and restart gdm."),
-				       GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("The gdm group does not exist. "
+		       "Please correct gdm configuration %s "
+		       "and restart gdm."),
+		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: Can't find the gdm group (%s). Aborting!"), GdmGroup);
+	    gdm_fail (_("%s: Can't find the gdm group (%s). Aborting!"), "gdm_config_parse", GdmGroup);
     } else  {
 	    GdmGroupId = grent->gr_gid;   
     }
 
     if (GdmGroupId == 0) {
-	    char *s = g_strdup_printf (_("The gdm group is set to be root, but "
-					 "this is not allowed since it can "
-					 "pose a security risk. Please "
-					 "correct gdm configuration %s and "
-					 "restart gdm."), GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("The gdm group is set to be root, but "
+		       "this is not allowed since it can "
+		       "pose a security risk. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: The gdm group should not be root. Aborting!"));
+	    gdm_fail (_("%s: The gdm group should not be root. Aborting!"), "gdm_config_parse");
     }
 
     setegid (GdmGroupId);	/* gid remains `gdm' */
@@ -540,12 +573,17 @@ gdm_config_parse (void)
 
     /* Check that the greeter can be executed */
     bin = ve_first_word (GdmGreeter);
-
     if (ve_string_empty (bin) ||
 	access (bin, X_OK) != 0) {
 	    gdm_error (_("%s: Greeter not found or can't be executed by the gdm user"), "gdm_config_parse");
     }
+    g_free (bin);
 
+    bin = ve_first_word (GdmRemoteGreeter);
+    if (ve_string_empty (bin) ||
+	access (bin, X_OK) != 0) {
+	    gdm_error (_("%s: Remote greeter not found or can't be executed by the gdm user"), "gdm_config_parse");
+    }
     g_free (bin);
 
 
@@ -563,52 +601,56 @@ gdm_config_parse (void)
 
     /* Enter paranoia mode */
     if (stat (GdmServAuthDir, &statbuf) == -1)  {
-	    char *s = g_strdup_printf (_("Server Authorization directory "
-					 "(daemon/ServAuthDir) is set to %s "
-					 "but this does not exist. Please "
-					 "correct gdm configuration %s and "
-					 "restart gdm."), GdmServAuthDir,
-				       GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but this does not exist. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GdmServAuthDir,
+		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: Authdir %s does not exist. Aborting."), GdmServAuthDir);
+	    gdm_fail (_("%s: Authdir %s does not exist. Aborting."), "gdm_config_parse", GdmServAuthDir);
     }
 
     if (! S_ISDIR (statbuf.st_mode)) {
-	    char *s = g_strdup_printf (_("Server Authorization directory "
-					 "(daemon/ServAuthDir) is set to %s "
-					 "but this is not a directory. Please "
-					 "correct gdm configuration %s and "
-					 "restart gdm."), GdmServAuthDir,
-				       GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but this is not a directory. Please "
+		       "correct gdm configuration %s and "
+		       "restart gdm."), GdmServAuthDir,
+		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: Authdir %s is not a directory. Aborting."), GdmServAuthDir);
+	    gdm_fail (_("%s: Authdir %s is not a directory. Aborting."), "gdm_config_parse", GdmServAuthDir);
     }
 
     if (statbuf.st_uid != GdmUserId || statbuf.st_gid != GdmGroupId)  {
-	    char *s = g_strdup_printf (_("Server Authorization directory "
-					 "(daemon/ServAuthDir) is set to %s "
-					 "but is not owned by user %s and group "
-					 "%s. Please correct the ownership or "
-					 "gdm configuration %s and restart "
-					 "gdm."),
-				       GdmServAuthDir, GdmUser, GdmGroup,
-				       GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but is not owned by user %s and group "
+		       "%s. Please correct the ownership or "
+		       "gdm configuration %s and restart "
+		       "gdm."),
+		     GdmServAuthDir, GdmUser, GdmGroup,
+		     GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: Authdir %s is not owned by user %s, group %s. Aborting."), 
+	    gdm_fail (_("%s: Authdir %s is not owned by user %s, group %s. Aborting."), "gdm_config_parse", 
 		      GdmServAuthDir, GdmUser, GdmGroup);
     }
 
     if (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRGRP|S_IXGRP))  {
-	    char *s = g_strdup_printf (_("Server Authorization directory "
-					 "(daemon/ServAuthDir) is set to %s "
-					 "but has the wrong permissions, it "
-					 "should have permissions of 0750. "
-					 "Please correct the permissions or "
-					 "the gdm configuration %s and "
-					 "restart gdm."),
-				       GdmServAuthDir, GDM_CONFIG_FILE);
+	    char *s = g_strdup_printf
+		    (_("Server Authorization directory "
+		       "(daemon/ServAuthDir) is set to %s "
+		       "but has the wrong permissions, it "
+		       "should have permissions of 0750. "
+		       "Please correct the permissions or "
+		       "the gdm configuration %s and "
+		       "restart gdm."),
+		     GdmServAuthDir, GDM_CONFIG_FILE);
 	    gdm_text_message_dialog (s);
-	    gdm_fail (_("gdm_config_parse: Authdir %s has wrong permissions %o. Should be 0750. Aborting."), 
+	    gdm_fail (_("%s: Authdir %s has wrong permissions %o. Should be 0750. Aborting."), "gdm_config_parse", 
 		      GdmServAuthDir, statbuf.st_mode);
     }
 
@@ -800,6 +842,8 @@ deal_with_x_crashes (GdmDisplay *d)
 		    char *xlog = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
 		    int ii;
 
+		    closelog ();
+
 		    for (ii = 0; ii < sysconf (_SC_OPEN_MAX); ii++)
 			    close (ii);
 
@@ -862,11 +906,12 @@ deal_with_x_crashes (GdmDisplay *d)
 	    /* Shit if we knew what the program was to tell the user,
 	     * the above script would have been defined and we'd run
 	     * it for them */
-	    char *error = _("I cannot start the X server (your graphical "
-			    "interface).  It is likely that it is not set "
-			    "up correctly. You will need to log in on a "
-			    "console and rerun the X configuration "
-			    "program.  Then restart GDM.");
+	    const char *error =
+		    _("I cannot start the X server (your graphical "
+		      "interface).  It is likely that it is not set "
+		      "up correctly. You will need to log in on a "
+		      "console and rerun the X configuration "
+		      "program.  Then restart GDM.");
 	    gdm_text_message_dialog (error);
     } /* else {
        * At this point .... screw the user, we don't know how to
@@ -1282,16 +1327,10 @@ main (int argc, char *argv[])
 
     store_argv (argc, argv);
 
-    setlocale(LC_ALL, "");
-    bindtextdomain(PACKAGE, GNOMELOCALEDIR);
-    textdomain(PACKAGE);
+    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+    textdomain (GETTEXT_PACKAGE);
 
-    /* XDM compliant error message */
-    if (getuid () != 0) {
-	    /* make sure the pid file doesn't get wiped */
-	    GdmPidFile = NULL;
-	    gdm_fail (_("Only root wants to run gdm\n"));
-    }
+    setlocale (LC_ALL, "");
 
     /* Initialize runtime environment */
     umask (022);
@@ -1303,6 +1342,13 @@ main (int argc, char *argv[])
 			GNOME_PARAM_CREATE_DIRECTORIES, FALSE,
 			NULL);
 
+    /* XDM compliant error message */
+    if (getuid () != 0) {
+	    /* make sure the pid file doesn't get wiped */
+	    GdmPidFile = NULL;
+	    gdm_fail (_("Only root wants to run gdm\n"));
+    }
+
     main_loop = g_main_loop_new (NULL, FALSE);
     openlog ("gdm", LOG_PID, LOG_DAEMON);
 
@@ -1383,7 +1429,7 @@ main (int argc, char *argv[])
     sigaddset (&mask, SIGCHLD);
     sigaddset (&mask, SIGHUP);
     sigaddset (&mask, SIGUSR1);
-    sigprocmask (SIG_UNBLOCK, &mask, &sysmask); /* Save system sigmask */
+    sigprocmask (SIG_UNBLOCK, &mask, NULL);
 
     gdm_debug ("gdm_main: Here we go...");
 
@@ -1511,6 +1557,20 @@ gdm_signal_notify (gint8 signal)
 	signals_notified[index] |= 1 << shift;
 }
 
+static void
+send_slave_ack (GdmDisplay *d)
+{
+	if (d->master_notify_fd >= 0) {
+		char not[2];
+		not[0] = GDM_SLAVE_NOTIFY_ACK;
+		not[1] = '\n';
+		write (d->master_notify_fd, not, 2);
+	}
+	if (d->slavepid > 0) {
+		kill (d->slavepid, SIGUSR2);
+	}
+}
+
 static void
 gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 {
@@ -1547,7 +1607,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->servpid = pid;
 			gdm_debug ("Got XPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_SESSPID " ",
 		            strlen (GDM_SOP_SESSPID " ")) == 0) {
@@ -1565,7 +1625,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->sesspid = pid;
 			gdm_debug ("Got SESSPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_GREETPID " ",
 		            strlen (GDM_SOP_GREETPID " ")) == 0) {
@@ -1583,7 +1643,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->greetpid = pid;
 			gdm_debug ("Got GREETPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_CHOOSERPID " ",
 		            strlen (GDM_SOP_CHOOSERPID " ")) == 0) {
@@ -1601,7 +1661,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->chooserpid = pid;
 			gdm_debug ("Got CHOOSERPID == %ld", (long)pid);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_LOGGED_IN " ",
 		            strlen (GDM_SOP_LOGGED_IN " ")) == 0) {
@@ -1627,7 +1687,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 				gdm_safe_restart ();
 
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_DISP_NUM " ",
 		            strlen (GDM_SOP_DISP_NUM " ")) == 0) {
@@ -1648,7 +1708,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->dispnum = disp_num;
 			gdm_debug ("Got DISP_NUM == %d", disp_num);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_VT_NUM " ",
 		            strlen (GDM_SOP_VT_NUM " ")) == 0) {
@@ -1669,7 +1729,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 #endif
 			gdm_debug ("Got VT_NUM == %d", vt_num);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_LOGIN " ",
 		            strlen (GDM_SOP_LOGIN " ")) == 0) {
@@ -1696,7 +1756,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->login = g_strdup (p);
 			gdm_debug ("Got LOGIN == %s", p);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_COOKIE " ",
 		            strlen (GDM_SOP_COOKIE " ")) == 0) {
@@ -1723,7 +1783,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->cookie = g_strdup (p);
 			gdm_debug ("Got COOKIE == <secret>");
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_FLEXI_ERR " ",
 		            strlen (GDM_SOP_FLEXI_ERR " ")) == 0) {
@@ -1760,7 +1820,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			
 			gdm_debug ("Got FLEXI_ERR == %d", err);
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strncmp (msg, GDM_SOP_FLEXI_OK " ",
 		            strlen (GDM_SOP_FLEXI_OK " ")) == 0) {
@@ -1790,13 +1850,45 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			
 			gdm_debug ("Got FLEXI_OK");
 			/* send ack */
-			kill (slave_pid, SIGUSR2);
+			send_slave_ack (d);
 		}
 	} else if (strcmp (msg, GDM_SOP_SOFT_RESTART) == 0) {
 		gdm_restart_mode = TRUE;
 		gdm_safe_restart ();
+	} else if (strncmp (msg, GDM_SOP_SYSLOG " ",
+		            strlen (GDM_SOP_SYSLOG " ")) == 0) {
+		char *p;
+		long pid;
+	       	int type;
+		p = strchr (msg, ' ');
+		if (p == NULL)
+			return;
+		p++;
+		if (sscanf (p, "%ld", &pid) != 1)
+			return;
+		p = strchr (p, ' ');
+		if (p == NULL)
+			return;
+		p++;
+		if (sscanf (p, "%d", &type) != 1)
+			return;
+
+		p = strchr (p, ' ');
+		if (p == NULL)
+			return;
+		p++;
+
+		syslog (type, "(child %ld) %s", pid, p);
 	} else if (strcmp (msg, GDM_SOP_START_NEXT_LOCAL) == 0) {
 		gdm_start_first_unborn_local (3 /* delay */);
+	} else if (strcmp (msg, GDM_SOP_HUP_ALL_GREETERS) == 0) {
+		/* probably shouldn't be done too often */
+		GSList *li;
+		for (li = displays; li != NULL; li = li->next) {
+			GdmDisplay *d = li->data;
+			if (d->greetpid > 0)
+				kill (d->greetpid, SIGHUP);
+		}
 	}
 }
 
@@ -2087,6 +2179,246 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 	/* Now we wait for the server to start up (or not) */
 }
 
+static gboolean
+is_key (const char *key1, const char *key2)
+{
+	char *key1d, *key2d, *p;
+
+	key1d = g_strdup (key1);
+	key2d = g_strdup (key2);
+
+	g_strstrip (key1d);
+	p = strchr (key1d, '=');
+	if (p != NULL)
+		*p = '\0';
+
+	g_strstrip (key2d);
+	p = strchr (key2d, '=');
+	if (p != NULL)
+		*p = '\0';
+
+	if (strcmp (key1d, key2d) == 0) {
+		g_free (key1d);
+		g_free (key2d);
+		return TRUE;
+	} else {
+		g_free (key1d);
+		g_free (key2d);
+		return FALSE;
+	}
+}
+
+static void
+notify_displays_int (const char *key, int val)
+{
+	GSList *li;
+	for (li = displays; li != NULL; li = li->next) {
+		GdmDisplay *disp = li->data;
+		if (disp->master_notify_fd >= 0) {
+			gdm_fdprintf (disp->master_notify_fd,
+				      "%c%s %d\n",
+				      GDM_SLAVE_NOTIFY_KEY, key, val);
+			if (disp->slavepid > 0)
+				kill (disp->slavepid, SIGUSR2);
+		}
+	}
+}
+
+static void
+notify_displays_string (const char *key, const char *val)
+{
+	GSList *li;
+	for (li = displays; li != NULL; li = li->next) {
+		GdmDisplay *disp = li->data;
+		if (disp->master_notify_fd >= 0) {
+			gdm_fdprintf (disp->master_notify_fd,
+				      "%c%s %s\n",
+				      GDM_SLAVE_NOTIFY_KEY, key, val);
+			if (disp->slavepid > 0)
+				kill (disp->slavepid, SIGUSR2);
+		}
+	}
+}
+
+static gboolean
+update_config (const char *key)
+{
+	struct stat statbuf;
+	if (stat (GDM_CONFIG_FILE, &statbuf) == -1) {
+		/* if the file didn't exist before either */
+		if (config_file_mtime == 0)
+			return TRUE;
+	} else {
+		/* if the file didn't exist before either */
+		if (config_file_mtime == statbuf.st_mtime)
+			return TRUE;
+		config_file_mtime = statbuf.st_mtime;
+	}
+
+	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
+
+	if (is_key (key, GDM_KEY_ALLOWROOT)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_ALLOWROOT);
+		if (ve_bool_equal (val, GdmAllowRoot))
+			goto update_config_ok;
+		GdmAllowRoot = val;
+
+		notify_displays_int (GDM_NOTIFY_ALLOWROOT, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_ALLOWREMOTEROOT)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_ALLOWREMOTEROOT);
+		if (ve_bool_equal (val, GdmAllowRemoteRoot))
+			goto update_config_ok;
+		GdmAllowRemoteRoot = val;
+
+		notify_displays_int (GDM_NOTIFY_ALLOWREMOTEROOT, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_ALLOWREMOTEAUTOLOGIN)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_ALLOWREMOTEAUTOLOGIN);
+		if (ve_bool_equal (val, GdmAllowRemoteAutoLogin))
+			goto update_config_ok;
+		GdmAllowRemoteAutoLogin = val;
+
+		notify_displays_int (GDM_NOTIFY_ALLOWREMOTEAUTOLOGIN, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_SYSMENU)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_SYSMENU);
+		if (ve_bool_equal (val, GdmSystemMenu))
+			goto update_config_ok;
+		GdmSystemMenu = val;
+
+		notify_displays_int (GDM_NOTIFY_SYSMENU, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_CONFIG_AVAILABLE)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
+		if (ve_bool_equal (val, GdmConfigAvailable))
+			goto update_config_ok;
+		GdmConfigAvailable = val;
+
+		notify_displays_int (GDM_NOTIFY_CONFIG_AVAILABLE, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_RETRYDELAY)) {
+		int val = gnome_config_get_int (GDM_KEY_RETRYDELAY);
+		if (val == GdmRetryDelay)
+			goto update_config_ok;
+		GdmRetryDelay = val;
+
+		notify_displays_int (GDM_NOTIFY_RETRYDELAY, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_GREETER)) {
+		char *val = gnome_config_get_string (GDM_KEY_GREETER);
+		if (strcmp (ve_sure_string (val), ve_sure_string (GdmGreeter)) == 0) {
+			g_free (val);
+			goto update_config_ok;
+		}
+		g_free (GdmGreeter);
+		GdmGreeter = val;
+
+		notify_displays_string (GDM_NOTIFY_GREETER, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_REMOTEGREETER)) {
+		char *val = gnome_config_get_string (GDM_KEY_REMOTEGREETER);
+		if (strcmp (ve_sure_string (val), ve_sure_string (GdmRemoteGreeter)) == 0) {
+			g_free (val);
+			goto update_config_ok;
+		}
+		g_free (GdmRemoteGreeter);
+		GdmRemoteGreeter = val;
+
+		notify_displays_string (GDM_NOTIFY_REMOTEGREETER, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_TIMED_LOGIN) ||
+		   is_key (key, GDM_KEY_TIMED_LOGIN_ENABLE)) {
+		gboolean enable = gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE);
+		char *val;
+
+		/* if not enabled, we just don't care */
+		if ( ! enable && ! GdmTimedLoginEnable)
+			goto update_config_ok;
+
+		val = gnome_config_get_string (GDM_KEY_TIMED_LOGIN);
+		if (strcmp (ve_sure_string (val),
+			    ve_sure_string (GdmTimedLogin)) == 0 &&
+		    ve_bool_equal (enable, GdmTimedLoginEnable)) {
+			g_free (val);
+			goto update_config_ok;
+		}
+
+		GdmTimedLoginEnable = enable;
+		g_free (GdmTimedLogin);
+		if (GdmTimedLoginEnable) {
+			GdmTimedLogin = val;
+		} else {
+			g_free (val);
+			GdmTimedLogin = NULL;
+		}
+
+		notify_displays_string (GDM_NOTIFY_TIMED_LOGIN,
+					ve_sure_string (GdmTimedLogin));
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_TIMED_LOGIN_DELAY)) {
+		int val = gnome_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+		if (val == GdmTimedLoginDelay)
+			goto update_config_ok;
+		GdmTimedLoginDelay = val;
+
+		notify_displays_int (GDM_NOTIFY_TIMED_LOGIN_DELAY, val);
+
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_XDMCP)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_XDMCP);
+		if (ve_bool_equal (val, GdmXdmcp))
+			goto update_config_ok;
+		GdmXdmcp = val;
+
+		if (GdmXdmcp) {
+			if (gdm_xdmcp_init ())
+				gdm_xdmcp_run ();
+		} else {
+			gdm_xdmcp_close ();
+		}
+
+		goto update_config_ok;
+	} else if (is_key (key, "xdmcp/PARAMETERS")) {
+		GdmDispPerHost = gnome_config_get_int (GDM_KEY_DISPERHOST);
+		GdmMaxPending = gnome_config_get_int (GDM_KEY_MAXPEND);
+		GdmMaxManageWait = gnome_config_get_int (GDM_KEY_MAXWAIT);
+		GdmMaxSessions = gnome_config_get_int (GDM_KEY_MAXSESS);
+		GdmIndirect = gnome_config_get_bool (GDM_KEY_INDIRECT);
+		GdmMaxIndirect = gnome_config_get_int (GDM_KEY_MAXINDIR);
+		GdmMaxIndirectWait = gnome_config_get_int (GDM_KEY_MAXINDWAIT);
+		GdmPingInterval = gnome_config_get_int (GDM_KEY_PINGINTERVAL);
+	} else if (is_key (key, GDM_KEY_UDPPORT)) {
+		int val = gnome_config_get_int (GDM_KEY_UDPPORT);
+		if (GdmPort == val)
+			goto update_config_ok;
+		GdmPort = val;
+
+		if (GdmXdmcp) {
+			gdm_xdmcp_close ();
+			if (gdm_xdmcp_init ())
+				gdm_xdmcp_run ();
+		}
+	}
+
+	gnome_config_pop_prefix ();
+	return FALSE;
+
+update_config_ok:
+
+	gnome_config_pop_prefix ();
+	return TRUE;
+}
+
 static void
 gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 {
@@ -2216,31 +2548,49 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 			GdmDisplay *disp = li->data;
 			if ( ! disp->console)
 				continue;
-			gdm_connection_write (conn, sep);
+			gdm_connection_printf (conn,
+					       "%s%s,%s,",
+					       sep,
+					       ve_sure_string (disp->name),
+					       ve_sure_string (disp->login));
 			sep = ";";
-			gdm_connection_write (conn,
-					      ve_sure_string (disp->name));
-			gdm_connection_write (conn, ",");
-			gdm_connection_write (conn,
-					      ve_sure_string (disp->login));
-			gdm_connection_write (conn, ",");
 			if (disp->type == TYPE_FLEXI_XNEST) {
 				gdm_connection_write
 					(conn,
 					 ve_sure_string (disp->xnest_disp));
 			} else {
-				char *vt = g_strdup_printf ("%d",
+				gdm_connection_printf (conn, "%d",
 #ifdef __linux__
-							    disp->vt
+						       disp->vt
 #else
-							    -1
+						       -1
 #endif
-							    );
-				gdm_connection_write (conn, vt);
-				g_free (vt);
+						      );
 			}
 		}
 		gdm_connection_write (conn, "\n");
+	} else if (strcmp (msg, GDM_SUP_GREETERPIDS) == 0) {
+		GSList *li;
+		const char *sep = " ";
+		gdm_connection_write (conn, "OK");
+		for (li = displays; li != NULL; li = li->next) {
+			GdmDisplay *disp = li->data;
+			gdm_connection_printf (conn, "%s%ld",
+					       sep, (long)disp->greetpid);
+			sep = ";";
+		}
+		gdm_connection_write (conn, "\n");
+	} else if (strncmp (msg, GDM_SUP_UPDATE_CONFIG " ",
+		     strlen (GDM_SUP_UPDATE_CONFIG " ")) == 0) {
+		const char *key = 
+			&msg[strlen (GDM_SUP_UPDATE_CONFIG " ")];
+
+		if ( ! update_config (key)) {
+			gdm_connection_write (conn,
+					      "ERROR 50 Unsupported key\n");
+		} else {
+			gdm_connection_write (conn, "OK\n");
+		}
 	} else if (strcmp (msg, GDM_SUP_VERSION) == 0) {
 		gdm_connection_write (conn, "GDM " VERSION "\n");
 	} else if (strcmp (msg, GDM_SUP_CLOSE) == 0) {
