@@ -61,6 +61,7 @@
 #define DISPLAY_CHOSEN 20	/* successful chooser session,
 				   restart display */
 #define DISPLAY_XFAILED 64	/* X failed */
+#define DISPLAY_RESTARTGREETER 127 /* Restart greeter */
 #define DISPLAY_RESTARTGDM 128	/* Restart GDM */
 
 enum {
@@ -104,6 +105,8 @@ enum {
 #define GDM_READPIC    '%' /* Send a user picture in a temp file */
 #define GDM_ERRBOX     'e' /* Puts string in the error box */
 #define GDM_ERRDLG     'E' /* Puts string up in an error dialog */
+#define GDM_NOFOCUS    'f' /* Don't focus the login window (optional) */
+#define GDM_FOCUS      'F' /* Allow focus on the login window again (optional) */
 
 /* Different login interruptions */
 #define GDM_INTERRUPT_TIMED_LOGIN 'T'
@@ -202,8 +205,6 @@ enum {
 #define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_GDMCONFIGDIR "/gdmconfig --disable-sound --disable-crash-dialog"
 #define GDM_KEY_CONFIG_AVAILABLE "greeter/ConfigAvailable=true"
 #define GDM_KEY_TITLE_BAR "greeter/TitleBar=true"
-/* translated string gets HATE defaults */
-#define GDM_KEY_WELCOME_TR "greeter/Welcome"
 #define GDM_KEY_WELCOME "greeter/Welcome=Welcome to %n"
 #define GDM_KEY_XINERAMASCREEN "greeter/XineramaScreen=0"
 #define GDM_KEY_BACKGROUNDPROG "greeter/BackgroundProgram="
@@ -217,6 +218,9 @@ enum {
 #define GDM_KEY_POSITIONX "greeter/PositionX=0"
 #define GDM_KEY_POSITIONY "greeter/PositionY=0"
 #define GDM_KEY_USE_24_CLOCK "greeter/Use24Clock=false"
+#define GDM_KEY_ENTRY_CIRCLES "greeter/UseCirclesInEntry=false"
+#define GDM_KEY_GRAPHICAL_THEME "greeter/GraphicalTheme=circles"
+#define GDM_KEY_GRAPHICAL_THEME_DIR "greeter/GraphicalThemeDir=" EXPANDED_DATADIR "/gdm/themes/"
 
 #define GDM_KEY_SCAN "chooser/ScanTime=3"
 #define GDM_KEY_HOST "chooser/DefaultHostImg=" EXPANDED_PIXMAPDIR "/nohost.png"
@@ -316,7 +320,7 @@ struct _GdmDisplay {
 
     /* Notification connection */
     int master_notify_fd;  /* write part of the connection */
-    GdmConnection *slave_notify_conn; /* read part of the connection */
+    int slave_notify_fd; /* read part of the connection */
 };
 
 typedef struct _GdmXServer GdmXServer;
@@ -417,14 +421,25 @@ GdmXServer *	gdm_find_x_server	(const char *id);
 #define GDM_SOP_SOFT_RESTART "SOFT_RESTART" /* no arguments */
 #define GDM_SOP_START_NEXT_LOCAL "START_NEXT_LOCAL" /* no arguments */
 #define GDM_SOP_HUP_ALL_GREETERS "HUP_ALL_GREETERS" /* no arguments */
+/* sometimes we can't do a syslog so we tell the main daemon */
+#define GDM_SOP_SYSLOG "SYSLOG" /* <pid> <type> <message> */
 
 /* Notification protocol */
 #define GDM_NOTIFY_ALLOWREMOTEROOT "AllowRemoteRoot" /* <true/false as int> */
 #define GDM_NOTIFY_ALLOWROOT "AllowRoot" /* <true/false as int> */
 #define GDM_NOTIFY_ALLOWREMOTEAUTOLOGIN "AllowRemoteAutoLogin" /* <true/false as int> */
+#define GDM_NOTIFY_SYSMENU "SystemMenu" /* <true/false as int> */
+#define GDM_NOTIFY_CONFIG_AVAILABLE "ConfigAvailable" /* <true/false as int> */
 #define GDM_NOTIFY_RETRYDELAY "RetryDelay" /* <seconds> */
 #define GDM_NOTIFY_GREETER "Greeter" /* <greeter binary> */
 #define GDM_NOTIFY_REMOTEGREETER "RemoteGreeter" /* <greeter binary> */
+#define GDM_NOTIFY_TIMED_LOGIN "TimedLogin" /* <login> */
+#define GDM_NOTIFY_TIMED_LOGIN_DELAY "TimedLoginDelay" /* <seconds> */
+
+/* Ack for a slave message */
+#define GDM_SLAVE_NOTIFY_ACK 'A'
+/* Update this key */
+#define GDM_SLAVE_NOTIFY_KEY '!'
 
 #define GDM_SUP_SOCKET "/tmp/.gdm_socket"
 
@@ -547,6 +562,11 @@ GdmXServer *	gdm_find_x_server	(const char *id);
  *			xdmcp/MaxPendingIndirect
  *			xdmcp/MaxWaitIndirect
  *			xdmcp/PingInterval (only affects new connections)
+ *   		 xdmcp/TimedLogin (2.3.90.3)
+ *   		 xdmcp/TimedLoginEnable (2.3.90.3)
+ *   		 xdmcp/TimedLoginDelay (2.3.90.3)
+ *   		 greeter/SystemMenu (2.3.90.3)
+ *   		 greeter/ConfigAvailable (2.3.90.3)
  * Supported since: 2.3.90.2
  * Arguments:  <key>
  *   <key> is just the base part of the key such as "security/AllowRemoteRoot"
