@@ -31,6 +31,7 @@
 #include <sys/wait.h>
 #include <strings.h>
 #include <signal.h>
+#include <syslog.h>
 #include <errno.h>
 #include <time.h>
 #include <X11/Xlib.h>
@@ -63,6 +64,7 @@ extern gboolean GdmXdmcp;
 extern gint high_display_num;
 extern pid_t extra_process;
 extern int extra_status;
+extern int gdm_in_signal;
 
 /* Global vars */
 static GdmDisplay *d = NULL;
@@ -164,17 +166,17 @@ busy_ask_user (GdmDisplay *disp)
     /* if we have "open" we can talk to the user */
     if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) == 0) {
 	    char *error = g_strdup_printf
-		    (gdm_cons_i18n (N_("There already appears to be an X server "
-				       "running on display %s.  Should I try another "
-				       "display number?  If you answer no, I will "
-				       "attempt to start the server on %s again.%s")),
+		    (_("There already appears to be an X server "
+		       "running on display %s.  Should I try another "
+		       "display number?  If you answer no, I will "
+		       "attempt to start the server on %s again.%s"),
 		     disp->name,
 		     disp->name,
 #ifdef __linux__
-		     gdm_cons_i18n (N_("  (You can change consoles by pressing Ctrl-Alt "
-				       "plus a function key, such as Ctrl-Alt-F7 to go "
-				       "to console 7.  X servers usually run on consoles "
-				       "7 and higher.)"))
+		     _("  (You can change consoles by pressing Ctrl-Alt "
+		       "plus a function key, such as Ctrl-Alt-F7 to go "
+		       "to console 7.  X servers usually run on consoles "
+		       "7 and higher.)")
 #else /* ! __linux__ */
 		     /* no info for non linux users */
 		     ""
@@ -665,6 +667,8 @@ gdm_server_spawn (GdmDisplay *d)
 	if (GdmXdmcp)
 		gdm_xdmcp_close();
 
+	closelog ();
+
 	/* close things */
 	for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
 		close(i);
@@ -675,6 +679,8 @@ gdm_server_spawn (GdmDisplay *d)
 	open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
 	open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
 
+	openlog ("gdm", LOG_PID, LOG_DAEMON);
+
 	/* Rotate the X server logs */
 	rotate_logs (d->name);
 
@@ -873,6 +879,8 @@ gdm_server_spawn (GdmDisplay *d)
 static void
 gdm_server_usr1_handler (gint sig)
 {
+    gdm_in_signal++;
+
     d->servstat = SERVER_RUNNING; /* Server ready to accept connections */
     d->starttime = time (NULL);
 
@@ -881,6 +889,8 @@ gdm_server_usr1_handler (gint sig)
     server_signal_notified = TRUE;
     /* this will quit the select */
     write (server_signal_pipe[1], "Yay!", 4);
+
+    gdm_in_signal--;
 }
 
 
@@ -894,6 +904,8 @@ gdm_server_usr1_handler (gint sig)
 static void 
 gdm_server_alarm_handler (gint signal)
 {
+    gdm_in_signal++;
+
     d->servstat = SERVER_TIMEOUT; /* Server didn't start */
 
     gdm_debug ("gdm_server_alarm_handler: Got SIGALRM, server abort");
@@ -901,6 +913,8 @@ gdm_server_alarm_handler (gint signal)
     server_signal_notified = TRUE;
     /* this will quit the select */
     write (server_signal_pipe[1], "Yay!", 4);
+
+    gdm_in_signal--;
 }
 
 
@@ -917,6 +931,8 @@ gdm_server_child_handler (int signal)
 	int status;
 	pid_t pid;
 
+	gdm_in_signal++;
+
 	gdm_debug ("gdm_server_child_handler: Got SIGCHLD");
 
 	while ((pid = waitpid (-1, &status, WNOHANG)) > 0) {
@@ -945,6 +961,8 @@ gdm_server_child_handler (int signal)
 			extra_status = status;
 		}
 	}
+
+	gdm_in_signal--;
 }
 
 
@@ -1005,7 +1023,7 @@ gdm_server_alloc (gint id, const gchar *command)
 
     d->timed_login_ok = FALSE;
 
-    d->slave_notify_conn = NULL;
+    d->slave_notify_fd = -1;
     d->master_notify_fd = -1;
     
     return d;
