@@ -195,8 +195,9 @@ main (int argc, char *argv[])
 	char **execvec;
 	pid_t pid;
 
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 
 	gnome_init_with_popt_table ("gdmXnestchooser", VERSION,
 				    argc, argv, options, 0, &ctx);
