@@ -1,11 +1,13 @@
-#define GREETER_KEY_THEME "greeter2/Theme=circles/circles.xml"
-#define GREETER_KEY_ENTRY_CIRCLES "greeter2/UseCirclesInEntry=false"
-
-extern int greeter_use_circles_in_entry;
 
+extern gboolean GdmUseCirclesInEntry;
 extern gboolean GdmShowGnomeChooserSession;
 extern gboolean GdmShowGnomeFailsafeSession;
 extern gboolean GdmShowXtermFailsafeSession;
 extern gboolean GdmShowLastSession;
+extern gboolean GdmSystemMenu;
+extern gboolean GdmConfigAvailable;
+extern gchar *GdmHalt;
+extern gchar *GdmReboot;
+extern gchar *GdmSuspend;
 extern gchar *GdmSessionDir;
 extern gchar *GdmDefaultLocale;
